public class FindRoots {
    /**
     * A simple example function f(x) = x^2 - x - 6.
     * Because f(x) = (x+2)(x-3), there are roots at
     * x == -2.0 and x == 3.0
     */
    public static double f(double x) {
        return x*x - x - 6;
    }

    /**
     * Find the root of the function f(x) with the range [a,b] using
     * a simple bisection method.
     * @param a minimum value of x to test
     * @param b maximum value of x to test
     * @param e tolerance - allowed size of interval within which root is found
     * @return x such that f(x+epsilon) == 0 where epsilon < e
     */
    public static double bisect(double a, double b, double e) {
        if (Math.signum(f(a)) == Math.signum(f(b))) throw new IllegalArgumentException("sign is same at both endpoints");
        
        while(Math.abs(b-a) >= e) {
           double m = (a+b) / 2.0;
           if (Math.signum(f(m)) == Math.signum(f(a))) {
               a = m;
           } else {
               b = m;
           }
        }
        return a;
    }

    public static void main(String[] args) {
        double a = 0.0;
        double b = 4.0;
        double e = 1.0e-15;
        double root = bisect(a, b, e);
        System.out.println(root); // should print something close to 3.0
    }
}
